import { jsx } from "react/jsx-runtime";
import React, { useRef, useDebugValue, useEffect, useCallback, createContext, useContext } from "react";
import { EditorView } from "@codemirror/view";
import { unstable_batchedUpdates } from "react-dom";
import { useSyncExternalStore } from "use-sync-external-store/shim";
function safeRequestAnimationFrame(callback) {
  if (window.requestAnimationFrame && window.self === window.top) {
    const requestId = window.requestAnimationFrame(callback);
    return () => window.cancelAnimationFrame(requestId);
  } else {
    const timeoutId = window.setTimeout(callback);
    return () => window.clearTimeout(timeoutId);
  }
}
function createAsyncScheduler() {
  let cancelLastRequest;
  return {
    request: (callback) => {
      cancelLastRequest = safeRequestAnimationFrame(() => {
        cancelLastRequest = void 0;
        callback();
      });
    },
    cancel: () => {
      if (cancelLastRequest) {
        cancelLastRequest();
        cancelLastRequest = void 0;
      }
    }
  };
}
function isFunction(value) {
  return typeof value === "function";
}
function useSingleton(instance) {
  const instanceRef = useRef(null);
  if (instanceRef.current === null) {
    instanceRef.current = isFunction(instance) ? instance() : instance;
  }
  return instanceRef.current;
}
const isBrowser = typeof window !== "undefined";
const builtInAPI = React.useInsertionEffect;
function syncFallback(effect, _deps) {
  effect();
}
const useInsertionEffectWithSyncFallback = isBrowser ? builtInAPI != null ? builtInAPI : syncFallback : syncFallback;
function useSyncedRef(value) {
  const ref = useRef(value);
  useInsertionEffectWithSyncFallback(() => {
    ref.current = value;
  }, [value]);
  return ref;
}
function createCodeMirror(config) {
  let prevState;
  function createConfig() {
    return (isFunction(config) ? config : () => config)(prevState);
  }
  let currentView;
  const viewUpdateCallbacks = /* @__PURE__ */ new Set();
  function subscribeViewUpdate(callback) {
    viewUpdateCallbacks.add(callback);
    return () => {
      viewUpdateCallbacks.delete(callback);
    };
  }
  function publishViewUpdate() {
    viewUpdateCallbacks.forEach((callback) => callback());
  }
  function setView(view) {
    if (view === currentView) {
      return;
    }
    if (currentView) {
      prevState = currentView.state;
      currentView.destroy();
    }
    currentView = view;
    publishViewUpdate();
  }
  const getView = () => currentView;
  const getServerView = () => void 0;
  const useView = () => {
    const view = useSyncExternalStore(subscribeViewUpdate, getView, getServerView);
    useDebugValue(view);
    return view;
  };
  const useViewEffect = (effect, deps) => {
    const view = useView();
    useEffect(() => {
      if (view) {
        return effect(view);
      }
    }, [view, ...deps]);
  };
  const useViewDispatch = (onViewNotReady) => {
    const viewNotReadyHandlerRef = useSyncedRef(onViewNotReady);
    return useCallback(
      (...specs) => {
        const view = getView();
        if (!view) {
          const fn = viewNotReadyHandlerRef.current;
          return fn == null ? void 0 : fn(...specs);
        }
        view.dispatch(...specs);
      },
      [viewNotReadyHandlerRef]
    );
  };
  let containerRef;
  function createContainerRef() {
    let currentContainer = null;
    const asyncScheduler = createAsyncScheduler();
    return {
      get current() {
        return currentContainer;
      },
      set current(container) {
        if (container === currentContainer) {
          return;
        }
        currentContainer = container;
        asyncScheduler.cancel();
        asyncScheduler.request(
          () => unstable_batchedUpdates(() => {
            setView(void 0);
            if (container) {
              const view = new EditorView({
                ...createConfig(),
                parent: container
              });
              setView(view);
            }
          })
        );
      }
    };
  }
  function getContainerRef() {
    return containerRef != null ? containerRef : containerRef = createContainerRef();
  }
  const useContainerRef = () => useSingleton(getContainerRef);
  return {
    getView,
    useView,
    useViewEffect,
    useViewDispatch,
    useContainerRef
  };
}
function toPascalCase(str) {
  return str.split(/[-_]/).map((word) => word[0].toUpperCase() + word.slice(1)).join("");
}
function createCodeMirrorWithContext(displayName) {
  const InternalCodeMirrorContext = createContext(null);
  const CodeMirrorProvider = ({ config, children }) => {
    const contextValue = useSingleton(() => createCodeMirror(config));
    return /* @__PURE__ */ jsx(InternalCodeMirrorContext.Provider, { value: contextValue, children });
  };
  if (displayName) {
    displayName = toPascalCase(displayName);
    CodeMirrorProvider.displayName = `${displayName}.Provider`;
    InternalCodeMirrorContext.displayName = `Internal${displayName}`;
  }
  const useCodeMirrorContext = () => {
    const contextValue = useContext(InternalCodeMirrorContext);
    if (!contextValue) {
      throw new Error(
        "could not find CodeMirrorContext value; please ensure the component is wrapped in a <Provider>"
      );
    }
    return contextValue;
  };
  const useView = () => {
    const { useView: useContextView } = useCodeMirrorContext();
    return useContextView();
  };
  const useViewEffect = (effect, deps) => {
    const { useViewEffect: useContextViewEffect } = useCodeMirrorContext();
    return useContextViewEffect(effect, deps);
  };
  const useViewDispatch = (onViewNotReady) => {
    const { useViewDispatch: useContextViewDispatch } = useCodeMirrorContext();
    return useContextViewDispatch(onViewNotReady);
  };
  const useContainerRef = () => {
    const { useContainerRef: useContextContainerRef } = useCodeMirrorContext();
    return useContextContainerRef();
  };
  return {
    Provider: CodeMirrorProvider,
    useContext: useCodeMirrorContext,
    useView,
    useViewEffect,
    useViewDispatch,
    useContainerRef
  };
}
export {
  createCodeMirror,
  createCodeMirrorWithContext
};
//# sourceMappingURL=index.js.map
